/**
Created by: Michael Hill <br><br>

KEYBOARD:<br>
'i' = Show/Hide Interface <br>
'h' = Show/Hide Nest Hull <br>
'd' = Switch between drawing mode and camera mode<br><br>

MOUSE:<br>
Scroll = Zoom<br>
Click + Drag = Rotate
**/

import processing.opengl.*;
import peasy.*;
import controlP5.*;

ControlP5 controlP5; //Allows controls to be created
ControlP5 instructions; //Draws instructions separately from the controls

PeasyCam cam; //Creates better camera control

PMatrix3D currCameraMatrix; //Holds camera matrix while drawing hud
PMatrix3D baseMatrix; //Stores inital camera position.  Used for drawing lights with peasycam
PGraphics3D g3; //Holds scene informaion so that camera can be reset
                //after drawing hud


//Allows individual to either rotate the object or draw it                
int drawingState = 0;
boolean drawHull = true;
boolean drawHUD = true;

//holds position of egg wrapping
float drawStep = 0;
float drawHeight = 0;

//Number of division lines to draw for the hull
int lineNum = 20;

//Location of points which control nest shape;
float Shape_Weight_1X = 15.0;
float Shape_Weight_1Y = 0.0;
float Shape_Weight_2X = 185.0;
float Shape_Weight_2Y = 40.0;

//Lenght of nest
float egg_length = 100.0;


//The egg object
Egg drawingEgg;

//Holds the allowable variation for color
int color_variation = 100;

void setup() {
  size(800, 600, OPENGL);
  frameRate(60);
  hint(ENABLE_OPENGL_2X_SMOOTH);
  g3 = (PGraphics3D)g;
  
  createInterface(); //Creates controlp5 interface
  
  //Sets Up Camera Control
  cam = new PeasyCam(this, 0, egg_length/2, 0, 100);
  cam.setMinimumDistance(200);
  cam.setMaximumDistance(800);
  
  drawingEgg = new Egg();
}

void draw(){
  /*
  currCameraMatrix = new PMatrix3D(g3.camera); //Store camera orentation
  camera(); //Clear camera
  directionalLight(51, 102, 126, 0, -1, 0);
  g3.camera = currCameraMatrix; //Reset Camera
  */
  
  background(50);
  
  if(mousePressed && drawingState == 1 && mouseY > 70 && mouseY <= height){
    drawWrapper(); 
  }
  if(drawHull){
    drawEggHull();
  }  
  drawingEgg.draw();
  
  drawHUD();
}

void drawEggHull(){
  noFill();
  strokeWeight(1.0);
  stroke(255,255,255,150);

  float verticalStep = egg_length/lineNum;
  float hullY = HALF_PI;
  
  for (int i = 0; i<lineNum; i++){
    float circleInc = TWO_PI/lineNum;
    float hullX = 0.0;
    float hullZ = HALF_PI;
    //Find radius for current circle
    float curveRadius =  bezierPoint(0, Shape_Weight_1X, Shape_Weight_2X, 0, i/float(lineNum));
    float curveHeightPosition = bezierPoint(0, Shape_Weight_1Y, Shape_Weight_2Y, egg_length, i/float(lineNum) );
    //float curveHeightPosition = egg_length/lineNum * i;
    
    //Draw current circle
    for (int j = 0; j<lineNum; j++){
      stroke(lerpColor(color(255,0,0), color(0,255,0), i/((float)lineNum)));
      line(sin(hullX-circleInc)*curveRadius, curveHeightPosition, sin(hullZ-circleInc)*curveRadius, sin(hullX)*curveRadius, curveHeightPosition, sin(hullZ)*curveRadius);
      hullX += circleInc;
      hullZ += circleInc;
    }
    stroke(255,255,255,150);
    
    //draw vertical hull curve
    pushMatrix();
    //line(sin(hullX-inc)*40.0, 0, sin(hullZ-inc)*40.0, sin(hullX)*40.0, 0, sin(hullZ)*40.0);  
    rotateY(hullY);
    bezier(0, 0, 0, Shape_Weight_1X, Shape_Weight_1Y, 0, Shape_Weight_2X, Shape_Weight_2Y, 0, 0, egg_length, 0);
    popMatrix();
    
    hullY += circleInc;
    //println(hullY);
  }
}



void drawHUD(){
  hint(DISABLE_DEPTH_TEST); //Disabable depth test to draw HUD over everything
  currCameraMatrix = new PMatrix3D(g3.camera); //Store camera orentation
  camera(); //Clear camera
  //Draw HUD
  if(drawHUD){
    drawInterfaceExtras(); 
    controlP5.draw();
  }
  
  g3.camera = currCameraMatrix; //Reset Camera
  hint(ENABLE_DEPTH_TEST); //Turn depth test back on

}


//Turn of camera control if using GUI or drawing
void mousePressed(){
  if(mouseY < 75 || drawingState == 1){
   cam.setMouseControlled(false);
  } 
}

//Turns camera control back on when mouse is released
void mouseReleased(){
    cam.setMouseControlled(true);
}


void keyPressed() {
  if (key == 'd') {
    if(drawingState == 0){
      drawingState = 1;
      cam.setMouseControlled(false);
      println("Drawing");
    }
    else{
       drawingState = 0;
       cam.setMouseControlled(true);
       println("Viewing");
    }
  }
  if (key == 'h'){
    drawHull = !drawHull;
    println("Draw egg hull: " + drawHull); 
  }
  if (key == 'i'){
    if (controlP5.isVisible()){
      controlP5.hide();
    }
    else{
       controlP5.show(); 
    }
    drawHUD = !drawHUD;
    println("Draw HUD: " + drawHUD);
  }
  if (key == 'c'){
    drawingEgg.clear(); 
  }
}

void drawWrapper(){
  //Finds ratio of mouse position to width for deciding how much angle to add to the rotational position
  //of the 3D cursor
  drawStep += float(mouseX)/float(width)+ 0.2;
  
  //prevents the circular position of the cursor from getting too big
  if (drawStep > TWO_PI){
   drawStep -= TWO_PI; 
  }
  
  //Find ratio of the y position of mouse to screen to determine where on the vertical 
  //curve the next point should go
  drawHeight = map(float(mouseY), 75, height, 0, 1);
  println(mouseY);
  
  float hullX = 0.0;
  float hullZ = HALF_PI;
  
  color newColor = findColor(drawHeight);
    
  //Find radius for current point
  float yPoint =  bezierPoint(0, Shape_Weight_1Y, Shape_Weight_2Y, egg_length, drawHeight);
  //Find x,y,z for next drawn point
  float xPoint =  sin(drawStep) * bezierPoint(0, Shape_Weight_1X, Shape_Weight_2X, 0, drawHeight);
  float zPoint =  sin(drawStep + HALF_PI) * bezierPoint(0, Shape_Weight_1X, Shape_Weight_2X, 0, drawHeight);
  PVector newPoint = new PVector(xPoint, yPoint, zPoint);
  drawingEgg.addPoint(newPoint, newColor);
}
